/* OS/2 REXX */
/*------------------------------------------------------------------------
 Simple pure-REXX file viewer. Also may be used with pipe redirection.
 File formats supported : text, DBF, OS/2 INI.
 Hex mode view supported.
 May launch editor (e.exe is default, may be changed easily).
 Looks good when used with File Commander as alternative viewer.
 Requires ANSI for color output and sometimes cursor positioning.
 Loads ENTIRE file into memory and then releases it, so be careful
   with large files! Pressing <ESC> during load stops loading and
   shows what's have been loaded at the moment.

 Usage sample : eye mydoc.txt
           Or : dir | eye

 ᬮ騪 䠩/STDIN,  ⮬ REXX'. 
  : ⥪⮢, DBF, OS/2 INI.
  HEX ० ᬮ.
   ᬮ ४  ࠭ DOS/Win/KOI8.
  । (e.exe).
 筮   騬 REXX 쬨.
  룫廊 ਪ祭  File Commander  Alt-F3.
 ॡ ANSI (梥  - ⢨  ).
 㧨  䠩    ⥬ 뢠 , ⠪   訬
   䠩 ஦!  <ESC>,  ⠭  㧪.
    ⮬ 㤥  ,  ᯥ 㧨.

 ਬ ᪠ : eye mydoc.txt
             : dir | eye


Author : Vladimir Kazimirchik (http://i.am/kzm, mailto:v_kazimirchik@yahoo.com)
FREE, on "AS IS" basis. No warranties at all.

-------------------------------------------------------------------------*/

  call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs

  Signal On Halt

  /*-----------------------------------*/
  aColor.1 = D2C(27) || '[1;36;44m'  /* default color */
  aColor.2 = D2C(27) || '[1;33;44m'  /* hilite color */
  aColor.3 = D2C(27) || '[1;33;41m'  /* alarm color */
  aColor.4 = D2C(27) || '[0;36;44m'  /* deleted record color (DBF) */
  aColor.white = D2C(27) || '[0;30;47m'
  aColor.black = D2C(27) || '[0;37;40m'
  /*-----------------------------------*/
  cPathTmp = Value('TMP', , 'OS2ENVIRONMENT')

  cSearch = ''
  lWrap = 0

  arg cFile .
  If cFile == '' Then Do

    If Lines() = 0 Then Do
      Say 'Usage : eye <filename>'
      Call SysGetKey 'noecho'
      Signal Halt
    End

    cFile = 'STDIN:'

  End

  Else Do
    Do While Chars() > 0
      Call SysGetKey 'noecho'   /* empty keyboard buffer */
    End
  End

  Call SysCurState 'Off'

  Parse Value SysTextScreenSize() With nRowsOld nCols

  /* == Uncomment this if you prefer to see many rows (I prefer 34) == 
  If nRowsOld < 34 Then Do
    nRows = 34
    'mode 80,34'
  End
  Else  */  nRows = nRowsOld

  Call CharOut , D2C(27) || '[' || nRows || ';1H'
       /* SysCurPos failed here after Alt-F3 in File Commander */

  Call CharOut , ,
         D2C(27) || '[0;37;40m' ||  '#' || aColor.white || '      ',
      || D2C(27) || '[0;37;40m' || ' 2' || aColor.white || '      ',
      || D2C(27) || '[0;37;40m' || ' 3' || aColor.white || 'C.page',
      || D2C(27) || '[0;37;40m' || ' 4' || aColor.white || 'Format',
      || D2C(27) || '[0;37;40m' || ' 5' || aColor.white || '      ',
      || D2C(27) || '[0;37;40m' || ' 6' || aColor.white || '      ',
      || D2C(27) || '[0;37;40m' || ' 7' || aColor.white || 'Search',
      || D2C(27) || '[0;37;40m' || ' 8' || aColor.white || '      ',
      || D2C(27) || '[0;37;40m' || ' 9' || aColor.white || 'Run E ',
      || D2C(27) || '[0;37;40m' || ' 10' || aColor.white || 'Quit '

  /* ⠥ । ⨯ 䠩 */

  nCurMode = 1 /* default - txt */

  cExt = Right(cFile, 4)   /* file extension */
  If cExt == '.DBF' Then nCurMode = 2
  If cExt == '.INI' Then nCurMode = 4
  If cExt == '.EXE' | cExt == '.COM' Then nCurMode = 3

  cModeName.1 = 'Text'
  cModeName.2 = 'DBF'
  cModeName.3 = 'Hex'
  cModeName.4 = 'OS2INI'

  Call FileLoad  /* load all file into memory and release it */

  Call CharOut , aColor.1

/*------------------------*/
  nUpper = 1
  If cExt == '.LOG' Then Do
    nUpper = aLine.0 - nRows + 4 /* directly to the end */
    If nUpper < 1 Then nUpper = 1
  End

  lRefresh = 1
  lEndShown = 0


  aPage.1 = '㪥뢠஫ᬨ'  ,
         || ''
  aPage.1.name = 'DOS'

  aPage.2 = '' || D2C(255) || '' ,
         || 'ި'
  aPage.2.name = 'WIN'

  aPage.3 = ''   ,
         || ''
  aPage.3.name = 'KOI-8'

  aPage.0 = 3
  nCurPage = 1

  nLeft = 1

/*------------------------*/

  Do Forever

    rc = syscurpos(0, 0)

    If lRefresh Then Do

      /*--upper line--*/
      Call SysCurPos 0, 0
      Call CharOut , aColor.white || ' ' ,
                       || Right(cFile  ,
                           || ' ' || Stream(cFile, 'c', 'query datetime') ,
                           || ' ' || Stream(cFile, 'c', 'query size') ,
                      , 64)  ,
                 || ' ' || Left(cModeName.nCurMode, nCols - 72)  ,
                 || Right(aPage.nCurPage.name, 5) || ' '


      Call CharOut , aColor.1

      Do i = 2 To nRows - 1

        Call SysCurPos i - 1, 0

        iTmp = nUpper + i - 2
        If iTmp <= aLine.0 Then Do

          cTmp = SubStr(aLine.iTmp, nLeft, nCols)
          If nCurPage \= 1 Then cTmp = Translate(cTmp, aPage.1, aPage.nCurPage)

          If aAttrib.iTmp \= 'AATTRIB.' || ITMP Then Do
            nAttrib = aAttrib.iTmp
            Call CharOut , aColor.nAttrib
          End

	  Call CharOut , Left(cTmp, nCols)
          Call CharOut , aColor.1
          lEndShown = 0
        End
        Else Do
          Call CharOut , Copies(' ', nCols)
          lEndShown = 1
        End
      End
    End

    lRefresh = 0

    nKey = c2d(SysGetKey('noecho'))

    If nKey = 27 Then Signal Halt

    If nKey = 0 | nKey = 224 then Do
      nKey2 = c2d(SysGetKey('noecho'))

      Select

        /* <F10> Quit */ When nKey = 0 & nKey2 = 68 Then Signal Halt

        /* <Down> */ When nKey = 224 & nKey2 = 80 & \lEndShown Then Do
          nUpper = nUpper + 1
          lRefresh = 1
        End

        /* <Up> */ When nKey = 224 & nKey2 = 72 & nUpper > 1 Then Do
          nUpper = nUpper - 1
          lRefresh = 1
        End

        /* <Right> */ When nKey = 224 & nKey2 = 77 & \lWrap Then Do
          nLeft = nLeft + 10
          lRefresh = 1
        End

        /* <Left> */ When nKey = 224 & nKey2 = 75 & nLeft > 10 Then Do
          nLeft = nLeft - 10
          lRefresh = 1
        End

        /* <PgDn> */ When nKey = 224 & nKey2 = 81 & \lEndShown Then Do
          nUpper = nUpper + nRows - 2
          lRefresh = 1
        End

        /* <PgUp> */ When nKey = 224 & nKey2 = 73 Then Do
          nUpper = nUpper - nRows + 2
          If nUpper < 1 Then nUpper = 1
          lRefresh = 1
        End

        /* <Home> */ When nKey = 224 & nKey2 = 71,
                    & (nUpper \= 1 | nLeft \= 1) Then Do
          nUpper = 1
          nLeft = 1
          lRefresh = 1
        End

        /* <End> */ When nKey = 224 & nKey2 = 79  ,
                         & (\lEndShown | nLeft \= 1) Then Do
          nUpper = aLine.0 - nRows + 4
          nLeft = 1
          lRefresh = 1
        End

        /* <F1> Help/Info */ When nKey = 0 & nKey2 = 59 Then Do
          Call SysCurPos 10, 20
          Call CharOut , aColor.White || '' || Copies('', 16)  ,
                         || ' Help ' || Copies('', 16) || ''
          Call SysCurPos 11, 20
          Call CharOut , aColor.White || '' || Copies(' ', 38) || ''
          Call SysCurPos 12, 20
          Call CharOut , aColor.White || '' || Copies(' ', 38) || ''
          Call SysCurPos 13, 20                                      
          Call CharOut , aColor.White || '' || Copies('', 38) || ''
        End


        /* <F2> Wrap/UnWrap */ When nKey = 0 & nKey2 = 60 ,
              & (nCurMode = 1 | nCurMode = 4) & cFile \== 'STDIN:'  ,
             Then Do
          lWrap = \lWrap
          Call FileLoad
          nLeft = 1
          If nUpper > aLine.0  /* ..  unwrap */ Then   ,
                  nUpper = aLine.0 - nRows + 4  /*  End */
          lRefresh = 1
        End

        /* <F3> Dos/Win/Koi */ When nKey = 0 & nKey2 = 61 Then Do
          nCurPage = nCurPage + 1
          If nCurPage > aPage.0 Then nCurPage = 1  /*   */
          lRefresh = 1
        End

        /* <F4> Text/DBF/Hex/INI */
        When nKey = 0 & nKey2 = 62 & cFile \== 'STDIN:' Then Do
          nCurMode = nCurMode + 1
          If nCurMode = 5 Then nCurMode = 1   /*   */
          Call FileLoad
          nUpper = 1
          nLeft = 1
          lRefresh = 1
        End

        /* <F7> Search */ When nKey = 0 & nKey2 = 65 Then Do

          Call SysCurPos 10, 10
          Call CharOut , aColor.White || '' || Copies('', 25)  ,
                         || ' Search ' || Copies('', 25) || ''
          Call SysCurPos 11, 10
          Call CharOut , aColor.White || '' || Copies(' ', 58) || ''
          Call SysCurPos 12, 10                                      
          Call CharOut , aColor.White || '' || Copies('', 58) || ''
          Call SysCurPos 11, 12
          Call CharOut , aColor.black || Copies(' ', 56)
          Call SysCurPos 11, 12
          Call SysCurState 'ON'

          Call CharOut , cSearch
          Do Forever

            nKeySearch = c2d(SysGetKey('noecho'))

            Select

              When nKeySearch = 27 Then Do
                Leave
              End

              When nKeySearch = 8 Then Do        /* backspace */
                If Length(cSearch) > 0 Then Do
                  cSearch = Left(cSearch, Length(cSearch) - 1)
                  Call CharOut , d2c(nKeySearch) || ' ' || d2c(nKeySearch)
                End
              End

              When nKeySearch = 13 Then Leave

              When nKeySearch = 0 | nKeySearch = 224 then ,  /* special keys */
                     Call SysGetKey 'noecho'

              OtherWise
                If Length(cSearch) < 55 Then Do
                  cSearch = cSearch || d2c(nKeySearch)
                  Call CharOut , d2c(nKeySearch)
                End

            End  /* select */

          End


          Call SysCurState 'OFF'

          /* search began */
          If nKeySearch \= 27 & cSearch \== '' Then Do

            cSearchUpper = Translate(cSearch)
            nSaveUpper = nUpper
            Do Forever
              nUpper = nUpper + 1
              If nUpper > aLine.0 Then Do  /* EOF */
                Call SysCurPos 11, 12
                Call CharOut , Left('Not Found', 56)
                Call SysGetKey 'noecho'
                nUpper = nSaveUpper
                Leave
              End
              If Pos(cSearchUpper, Translate(aLine.nUpper)) \= 0 Then Leave
            End
          End

          lRefresh = 1
        End

        /* <F9> Run E of current image */ When nKey = 0 & nKey2 = 67 Then Do
          cFileOut = SysTempFileName(cPathTmp || '\eye?????.tmp')

          Do iTmp = 1 To aLine.0 - 1   /* 'EOF' away */

            /* not include field header(s) in DBF */
            If nCurMode = 2 /* DBF */ & aAttrib.iTmp = 2 /* hilited */ ,
               Then Iterate

            cTmp = aLine.iTmp

            If nCurPage \= 1 Then  ,
                    cTmp = Translate(cTmp, aPage.1, aPage.nCurPage)
            Call LineOut cFileOut, cTmp

          End

          Call LineOut cFileOut  /* close */
          '@echo off'
          'Start /pm /f e ' || cFileOut
          Call SysSleep 1

        End

        OtherWise NOP
      End

    End

  End

halt:

Call SysFileTree cPathTmp || '\eye?????.tmp', 'aFile.', 'FO'
Do i = 1 To aFile.0
  Call SysFileDelete aFile.i
End

/* == Uncomment this if you used 'mode' command at the beginning ==
If nRowsOld < 34 Then 'mode 80,' || nRowsOld */

Call SysCls  

Exit

/*===========================================================*/
FileLoad:Procedure Expose cFile aLine. aAttrib. nCurMode nRows nCols  ,
                          aColor. cModeName. lWrap

  Drop aLine. aAttrib.

  Call SysCurPos nRows - 1, 1
  Call CharOut , aColor.white || '      '

  Call SysCurPos nRows - 1, 9
  If lWrap Then Call CharOut , aColor.white || 'UnWrap'
  Else Call CharOut , aColor.white || 'Wrap  '

  cTabHex = ''
  Do i = 1 To 32
    cTabHex = cTabHex || D2C(i - 1)
  End

  If Stream(cFile, 'c', 'query exists') == '' Then Do
    aLine.1 = ' ' || cFile || '  '
    aLine.0 = 1
    aAttrib.1 = 3
    Return
  End

  If Left(Stream(cFile, 'c', 'open read'), 9) == 'NOTREADY:' Then Do
    aLine.1 = ' 㤠  䠩 ' || cFile
    aLine.0 = 1
    aAttrib.1 = 3
    Return
  End


  Do Forever   /*  横 ॡࠥ  㯭 ਠ */

    Select   /*-----------different modes-------------*/

      When nCurMode = 1 Then Do  /*-----Text mode-----*/

        i = 0
        Do While Lines(cFile) > 0

          If cFile \== 'STDIN:' & Chars() Then Do
            nKey = C2D(SysGetKey('noecho'))
            If nKey = 27 Then Do
              Call Stream cFile, 'c', 'close'
              i = i + 1
              aLine.i = 'Transfer interrupted'
              aAttrib.i = 3
              aLine.0 = i
              Return
            End
          End

          i = i + 1
          Call SysCurPos nRows - 1, 1
          Call CharOut , aColor.white || Left(i, 6)

          aLine.i = Translate(LineIn(cFile), Copies('.', 32), cTabHex)

          If lWrap Then Do
            Do While Length(aLine.i) > nCols

              j = i + 1
              aLine.j = SubStr(aLine.i, nCols + 1)
              aLine.i = SubStr(aLine.i, 1, nCols)
              i = i + 1

            End
          End

        End
      End

      When nCurMode = 2 Then Do  /*-----DBF mode-----*/

        nSign = C2D(CharIn(cFile, 1, 1))

        If nSign \= 3 & nSign \= 131 Then Do
          nCurMode = 3   /*  DBF (DBaseIII), ஡㥬 hex */
          Iterate
        End

        nHRecs = c2d(Reverse(CharIn(cFile, 5, 4)))
        nHSize = c2d(Reverse(CharIn(cFile, 9, 2)))
        nRecSize = c2d(Reverse(CharIn(cFile, 11, 2)))

        /* structure */
        cStruLine = ''

        nPoint = 33
        i = 0
        Do While nPoint < (nHSize - 5)
          i = i + 1
          cTmp = CharIn(cFile, nPoint, 16)
          aStruct.i.field_name = SubStr(cTmp, 1, Pos('00'x, cTmp) - 1)
          aStruct.i.field_type = SubStr(cTmp, 12, 1)
          nPoint = nPoint + 16
          cTmp = CharIn(cFile, nPoint, 16)
          aStruct.i.field_len = c2d(SubStr(cTmp, 1, 1))
          aStruct.i.field_dec = c2d(SubStr(cTmp, 2, 1))

          /*-field representation in view--*/
          aStruct.i.field_def = aStruct.i.field_name || '/'  ,
                   || aStruct.i.field_type
          Select

            When aStruct.i.field_type == 'C' Then  ,
               aStruct.i.field_def = aStruct.i.field_def || aStruct.i.field_len

            When aStruct.i.field_type == 'N' Then  ,
               aStruct.i.field_def = aStruct.i.field_def || aStruct.i.field_len,
                                     || '.' || aStruct.i.field_dec
            OtherWise NOP
          End

          cStruLine = cStruLine || Left(aStruct.i.field_def,  ,
                      Max(Length(aStruct.i.field_def), aStruct.i.field_len)) ,
                      || ' '

          nPoint = nPoint + 16
        End

        nFields = i

        nPoint = nHSize + 1
        cTmp = CharIn(cFile, nPoint, 0)

        i = 0
        nRecs = 0

        Do Forever
          cTmp = Translate(CharIn(cFile, , nRecSize), Copies('.', 32), cTabHex)
          If Length(cTmp) < nRecSize Then Leave

          If cFile \== 'STDIN:' & Chars() Then Do
            nKey = C2D(SysGetKey('noecho'))
            If nKey = 27 Then Do
              Call Stream cFile, 'c', 'close'
              i = i + 1
              aLine.i = 'Transfer interrupted'
              aAttrib.i = 3
              aLine.0 = i
              Return
            End
          End

          i = i + 1
          nRecs = nRecs + 1
          Call SysCurPos nRows - 1, 1
          Call CharOut , aColor.white || Left(nRecs, 6)

          If i // (nRows - 2) = 1 Then Do
            aLine.i = cStruLine
            aAttrib.i = 2
            i = i + 1
          End

          If Left(cTmp, 1) == '*' Then aAttrib.i = 4

          aLine.i = ''

          nTmp = 2
          Do j = 1 To nFields

            If aStruct.j.field_type = 'M' Then cVar = '  <Memo>  '

            Else Do
              cVar = SubStr(cTmp, nTmp, aStruct.j.field_len)

              If aStruct.j.field_type = 'D' Then cVar = SubStr(cVar, 7, 2)  ,
                        || '/' || SubStr(cVar, 5, 2) || '/' || SubStr(cVar, 3, 2)
            End

            nTmp = nTmp + aStruct.j.field_len
            aLine.i = aLine.i || Left(cVar,  ,
                      Max(Length(aStruct.j.field_def), Length(cVar))) ,
                    || ' '
          End

        End

      End

      When nCurMode = 3 Then Do  /*-----Hex mode-----*/
/*
00020  3F 3F 2E 2E  2E 20 20 2A  2F 0D 0A 2F  2A 20 72 65   ??...  ./../. re
00030  71 75 69 72  65 73 20 52  65 78 78 55  74 69 6C 2E   quires RexxUtil.
right(d2x((i - 1)*16), 5, '0')
*/
        i = 0
        Do Forever

          cTmp = CharIn(cFile, , 16)
          If Length(cTmp) = 0 Then Leave

          If cFile \== 'STDIN:' & Chars() Then Do
            nKey = C2D(SysGetKey('noecho'))
            If nKey = 27 Then Do
              Call Stream cFile, 'c', 'close'
              i = i + 1
              aLine.i = 'Transfer interrupted'
              aAttrib.i = 3
              aLine.0 = i
              Return
            End
          End

          i = i + 1
          aLine.i = right(d2x((i - 1)*16), 5, '0') || ' '

          Call SysCurPos nRows - 1, 1
          Call CharOut , aColor.white || aLine.i

          Do j = 1 To Length(cTmp)
            If j // 4 = 1 Then aLine.i = aLine.i || ' |'
            aLine.i = aLine.i || ' ' || c2x(SubStr(cTmp, j, 1))
          End

          aLine.i = Left(aLine.i, 63)   ,
                        || Translate(cTmp,  Copies('.', 32), cTabHex)

          If Length(cTmp) < 16 Then Leave
        End

      End


      When nCurMode = 4 Then Do  /*-----INI mode-----*/

        rc = SysIni(cFile, 'all:', 'aProg.')

        If aProg.0 = 'APROG.0' Then Do
          nCurMode = 1   /*  INI (OS/2), ஡㥬 text */
          Iterate
        End

        i = 0
        Do ii = 1 to aProg.0

          If cFile \== 'STDIN:' & Chars() Then Do
            nKey = C2D(SysGetKey('noecho'))
            If nKey = 27 Then Do
              Call Stream cFile, 'c', 'close'
              i = i + 1
              aLine.i = 'Transfer interrupted'
              aAttrib.i = 3
              aLine.0 = i
              Return
            End
          End

          i = i + 1
          aLine.i = '[' || aProg.ii || ']'
          aAttrib.i = 2

          Call SysCurPos nRows - 1, 1      /* ii : - ᥪ権 */
          Call CharOut , aColor.white || Left(ii, 6)

          Call SysIni cFile, aProg.ii, 'all:', 'aItem.'
          Do j = 1 To aItem.0
            i = i + 1
            aLine.i = Translate(aItem.j || '='  ,
                || SysIni(cFile, aProg.ii, aItem.j), Copies('.', 32), cTabHex)

            If lWrap Then Do
              Do While Length(aLine.i) > nCols

                j = i + 1
                aLine.j = SubStr(aLine.i, nCols + 1)
                aLine.i = SubStr(aLine.i, 1, nCols)
                i = i + 1

              End
            End

          End
        End

      End

    OtherWise NOP

    End /* select */

    Leave

  End /* Do Forever */

  Call SysCurPos 1, 66

  Call Stream cFile, 'c', 'close'

  i = i + 1
  aLine.i = '>>EOF<<'
  aAttrib.i = 2

  aLine.0 = i

Return
